@@ -4,7 +4,6 @@
 // Author:      Julian Smart
 // Modified by: Vadim Zeitlin to be less MSW-specific on 10/10/98
 // Created:     01/02/97
-// RCS-ID:      $Id$
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -34,17 +33,9 @@ class  WXDLLIMPEXP_FWD_CORE wxImageList;
 class  WXDLLIMPEXP_FWD_CORE wxDragImage;
 struct WXDLLIMPEXP_FWD_CORE wxTreeViewItem;
 
-#if WXWIN_COMPATIBILITY_2_6
-    // NB: all the following flags are for compatbility only and will be removed in the
-    //     next versions
-    // flags for deprecated InsertItem() variant (their values are the same as of
-    // TVI_FIRST and TVI_LAST)
-    #define wxTREE_INSERT_FIRST 0xFFFF0001
-    #define wxTREE_INSERT_LAST  0xFFFF0002
-#endif
-
 // hash storing attributes for our items
-WX_DECLARE_EXPORTED_VOIDPTR_HASH_MAP(wxTreeItemAttr *, wxMapTreeAttr);
+class wxItemAttr;
+WX_DECLARE_EXPORTED_VOIDPTR_HASH_MAP(wxItemAttr *, wxMapTreeAttr);
 
 // ----------------------------------------------------------------------------
 // wxTreeCtrl
@@ -62,7 +53,7 @@ public:
                const wxSize& size = wxDefaultSize,
                long style = wxTR_HAS_BUTTONS | wxTR_LINES_AT_ROOT,
                const wxValidator& validator = wxDefaultValidator,
-               const wxString& name = wxTreeCtrlNameStr)
+               const wxString& name = wxASCII_STR(wxTreeCtrlNameStr))
     {
         Create(parent, id, pos, size, style, validator, name);
     }
@@ -74,122 +65,122 @@ public:
                 const wxSize& size = wxDefaultSize,
                 long style = wxTR_HAS_BUTTONS | wxTR_LINES_AT_ROOT,
                 const wxValidator& validator = wxDefaultValidator,
-                const wxString& name = wxTreeCtrlNameStr);
+                const wxString& name = wxASCII_STR(wxTreeCtrlNameStr));
 
     // implement base class pure virtuals
     // ----------------------------------
 
-    virtual unsigned int GetCount() const;
+    virtual unsigned int GetCount() const wxOVERRIDE;
 
-    virtual unsigned int GetIndent() const;
-    virtual void SetIndent(unsigned int indent);
+    virtual unsigned int GetIndent() const wxOVERRIDE;
+    virtual void SetIndent(unsigned int indent) wxOVERRIDE;
 
-    virtual void SetImageList(wxImageList *imageList);
-    virtual void SetStateImageList(wxImageList *imageList);
+    virtual void SetImageList(wxImageList *imageList) wxOVERRIDE;
+    virtual void SetStateImageList(wxImageList *imageList) wxOVERRIDE;
 
-    virtual wxString GetItemText(const wxTreeItemId& item) const;
+    virtual wxString GetItemText(const wxTreeItemId& item) const wxOVERRIDE;
     virtual int GetItemImage(const wxTreeItemId& item,
-                        wxTreeItemIcon which = wxTreeItemIcon_Normal) const;
-    virtual wxTreeItemData *GetItemData(const wxTreeItemId& item) const;
-    virtual wxColour GetItemTextColour(const wxTreeItemId& item) const;
-    virtual wxColour GetItemBackgroundColour(const wxTreeItemId& item) const;
-    virtual wxFont GetItemFont(const wxTreeItemId& item) const;
+                        wxTreeItemIcon which = wxTreeItemIcon_Normal) const wxOVERRIDE;
+    virtual wxTreeItemData *GetItemData(const wxTreeItemId& item) const wxOVERRIDE;
+    virtual wxColour GetItemTextColour(const wxTreeItemId& item) const wxOVERRIDE;
+    virtual wxColour GetItemBackgroundColour(const wxTreeItemId& item) const wxOVERRIDE;
+    virtual wxFont GetItemFont(const wxTreeItemId& item) const wxOVERRIDE;
 
-    virtual void SetItemText(const wxTreeItemId& item, const wxString& text);
+    virtual void SetItemText(const wxTreeItemId& item, const wxString& text) wxOVERRIDE;
     virtual void SetItemImage(const wxTreeItemId& item, int image,
-                      wxTreeItemIcon which = wxTreeItemIcon_Normal);
-    virtual void SetItemData(const wxTreeItemId& item, wxTreeItemData *data);
-    virtual void SetItemHasChildren(const wxTreeItemId& item, bool has = true);
-    virtual void SetItemBold(const wxTreeItemId& item, bool bold = true);
+                      wxTreeItemIcon which = wxTreeItemIcon_Normal) wxOVERRIDE;
+    virtual void SetItemData(const wxTreeItemId& item, wxTreeItemData *data) wxOVERRIDE;
+    virtual void SetItemHasChildren(const wxTreeItemId& item, bool has = true) wxOVERRIDE;
+    virtual void SetItemBold(const wxTreeItemId& item, bool bold = true) wxOVERRIDE;
     virtual void SetItemDropHighlight(const wxTreeItemId& item,
-                                      bool highlight = true);
+                                      bool highlight = true) wxOVERRIDE;
     virtual void SetItemTextColour(const wxTreeItemId& item,
-                                   const wxColour& col);
+                                   const wxColour& col) wxOVERRIDE;
     virtual void SetItemBackgroundColour(const wxTreeItemId& item,
-                                         const wxColour& col);
-    virtual void SetItemFont(const wxTreeItemId& item, const wxFont& font);
+                                         const wxColour& col) wxOVERRIDE;
+    virtual void SetItemFont(const wxTreeItemId& item, const wxFont& font) wxOVERRIDE;
 
     // item status inquiries
     // ---------------------
 
-    virtual bool IsVisible(const wxTreeItemId& item) const;
-    virtual bool ItemHasChildren(const wxTreeItemId& item) const;
-    virtual bool IsExpanded(const wxTreeItemId& item) const;
-    virtual bool IsSelected(const wxTreeItemId& item) const;
-    virtual bool IsBold(const wxTreeItemId& item) const;
+    virtual bool IsVisible(const wxTreeItemId& item) const wxOVERRIDE;
+    virtual bool ItemHasChildren(const wxTreeItemId& item) const wxOVERRIDE;
+    virtual bool IsExpanded(const wxTreeItemId& item) const wxOVERRIDE;
+    virtual bool IsSelected(const wxTreeItemId& item) const wxOVERRIDE;
+    virtual bool IsBold(const wxTreeItemId& item) const wxOVERRIDE;
 
     virtual size_t GetChildrenCount(const wxTreeItemId& item,
-                                    bool recursively = true) const;
+                                    bool recursively = true) const wxOVERRIDE;
 
     // navigation
     // ----------
 
-    virtual wxTreeItemId GetRootItem() const;
-    virtual wxTreeItemId GetSelection() const;
-    virtual size_t GetSelections(wxArrayTreeItemIds& selections) const;
-    virtual wxTreeItemId GetFocusedItem() const;
+    virtual wxTreeItemId GetRootItem() const wxOVERRIDE;
+    virtual wxTreeItemId GetSelection() const wxOVERRIDE;
+    virtual size_t GetSelections(wxArrayTreeItemIds& selections) const wxOVERRIDE;
+    virtual wxTreeItemId GetFocusedItem() const wxOVERRIDE;
 
-    virtual void ClearFocusedItem();
-    virtual void SetFocusedItem(const wxTreeItemId& item);
+    virtual void ClearFocusedItem() wxOVERRIDE;
+    virtual void SetFocusedItem(const wxTreeItemId& item) wxOVERRIDE;
 
 
-    virtual wxTreeItemId GetItemParent(const wxTreeItemId& item) const;
+    virtual wxTreeItemId GetItemParent(const wxTreeItemId& item) const wxOVERRIDE;
     virtual wxTreeItemId GetFirstChild(const wxTreeItemId& item,
-                                       wxTreeItemIdValue& cookie) const;
+                                       wxTreeItemIdValue& cookie) const wxOVERRIDE;
     virtual wxTreeItemId GetNextChild(const wxTreeItemId& item,
-                                      wxTreeItemIdValue& cookie) const;
-    virtual wxTreeItemId GetLastChild(const wxTreeItemId& item) const;
+                                      wxTreeItemIdValue& cookie) const wxOVERRIDE;
+    virtual wxTreeItemId GetLastChild(const wxTreeItemId& item) const wxOVERRIDE;
 
-    virtual wxTreeItemId GetNextSibling(const wxTreeItemId& item) const;
-    virtual wxTreeItemId GetPrevSibling(const wxTreeItemId& item) const;
+    virtual wxTreeItemId GetNextSibling(const wxTreeItemId& item) const wxOVERRIDE;
+    virtual wxTreeItemId GetPrevSibling(const wxTreeItemId& item) const wxOVERRIDE;
 
-    virtual wxTreeItemId GetFirstVisibleItem() const;
-    virtual wxTreeItemId GetNextVisible(const wxTreeItemId& item) const;
-    virtual wxTreeItemId GetPrevVisible(const wxTreeItemId& item) const;
+    virtual wxTreeItemId GetFirstVisibleItem() const wxOVERRIDE;
+    virtual wxTreeItemId GetNextVisible(const wxTreeItemId& item) const wxOVERRIDE;
+    virtual wxTreeItemId GetPrevVisible(const wxTreeItemId& item) const wxOVERRIDE;
 
     // operations
     // ----------
 
     virtual wxTreeItemId AddRoot(const wxString& text,
                          int image = -1, int selectedImage = -1,
-                         wxTreeItemData *data = NULL);
+                         wxTreeItemData *data = NULL) wxOVERRIDE;
 
-    virtual void Delete(const wxTreeItemId& item);
-    virtual void DeleteChildren(const wxTreeItemId& item);
-    virtual void DeleteAllItems();
+    virtual void Delete(const wxTreeItemId& item) wxOVERRIDE;
+    virtual void DeleteChildren(const wxTreeItemId& item) wxOVERRIDE;
+    virtual void DeleteAllItems() wxOVERRIDE;
 
-    virtual void Expand(const wxTreeItemId& item);
-    virtual void Collapse(const wxTreeItemId& item);
-    virtual void CollapseAndReset(const wxTreeItemId& item);
-    virtual void Toggle(const wxTreeItemId& item);
+    virtual void Expand(const wxTreeItemId& item) wxOVERRIDE;
+    virtual void Collapse(const wxTreeItemId& item) wxOVERRIDE;
+    virtual void CollapseAndReset(const wxTreeItemId& item) wxOVERRIDE;
+    virtual void Toggle(const wxTreeItemId& item) wxOVERRIDE;
 
-    virtual void Unselect();
-    virtual void UnselectAll();
-    virtual void SelectItem(const wxTreeItemId& item, bool select = true);
-    virtual void SelectChildren(const wxTreeItemId& parent);
+    virtual void Unselect() wxOVERRIDE;
+    virtual void UnselectAll() wxOVERRIDE;
+    virtual void SelectItem(const wxTreeItemId& item, bool select = true) wxOVERRIDE;
+    virtual void SelectChildren(const wxTreeItemId& parent) wxOVERRIDE;
 
-    virtual void EnsureVisible(const wxTreeItemId& item);
-    virtual void ScrollTo(const wxTreeItemId& item);
+    virtual void EnsureVisible(const wxTreeItemId& item) wxOVERRIDE;
+    virtual void ScrollTo(const wxTreeItemId& item) wxOVERRIDE;
 
     virtual wxTextCtrl *EditLabel(const wxTreeItemId& item,
-                          wxClassInfo* textCtrlClass = wxCLASSINFO(wxTextCtrl));
-    virtual wxTextCtrl *GetEditControl() const;
+                          wxClassInfo* textCtrlClass = wxCLASSINFO(wxTextCtrl)) wxOVERRIDE;
+    virtual wxTextCtrl *GetEditControl() const wxOVERRIDE;
     virtual void EndEditLabel(const wxTreeItemId& WXUNUSED(item),
-                              bool discardChanges = false)
+                              bool discardChanges = false) wxOVERRIDE
     {
         DoEndEditLabel(discardChanges);
     }
 
-    virtual void SortChildren(const wxTreeItemId& item);
+    virtual void SortChildren(const wxTreeItemId& item) wxOVERRIDE;
 
     virtual bool GetBoundingRect(const wxTreeItemId& item,
                                  wxRect& rect,
-                                 bool textOnly = false) const;
+                                 bool textOnly = false) const wxOVERRIDE;
 
     // implementation
     // --------------
 
-    virtual wxVisualAttributes GetDefaultAttributes() const
+    virtual wxVisualAttributes GetDefaultAttributes() const wxOVERRIDE
     {
         return GetClassDefaultAttributes(GetWindowVariant());
     }
@@ -198,20 +189,31 @@ public:
     GetClassDefaultAttributes(wxWindowVariant variant = wxWINDOW_VARIANT_NORMAL);
 
 
-    virtual WXLRESULT MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lParam);
-    virtual WXLRESULT MSWDefWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lParam);
-    virtual bool MSWCommand(WXUINT param, WXWORD id);
-    virtual bool MSWOnNotify(int idCtrl, WXLPARAM lParam, WXLPARAM *result);
-    virtual bool MSWShouldPreProcessMessage(WXMSG* msg);
+    virtual WXLRESULT MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lParam) wxOVERRIDE;
+    virtual WXLRESULT MSWDefWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lParam) wxOVERRIDE;
+    virtual bool MSWCommand(WXUINT param, WXWORD id) wxOVERRIDE;
+    virtual bool MSWOnNotify(int idCtrl, WXLPARAM lParam, WXLPARAM *result) wxOVERRIDE;
+    virtual bool MSWShouldPreProcessMessage(WXMSG* msg) wxOVERRIDE;
 
     // override some base class virtuals
-    virtual bool SetBackgroundColour(const wxColour &colour);
-    virtual bool SetForegroundColour(const wxColour &colour);
+    virtual bool SetBackgroundColour(const wxColour &colour) wxOVERRIDE;
+    virtual bool SetForegroundColour(const wxColour &colour) wxOVERRIDE;
 
     // returns true if the platform should explicitly apply a theme border
-    virtual bool CanApplyThemeBorder() const { return false; }
+    virtual bool CanApplyThemeBorder() const wxOVERRIDE { return false; }
+
+    virtual bool IsDoubleBuffered() const wxOVERRIDE;
+    virtual void SetDoubleBuffered(bool on) wxOVERRIDE;
 
 protected:
+    // Implement "update locking" in a custom way for this control.
+    virtual void DoFreeze() wxOVERRIDE;
+    virtual void DoThaw() wxOVERRIDE;
+
+    virtual bool MSWShouldSetDefaultFont() const wxOVERRIDE { return false; }
+
+    virtual void OnImagesChanged() wxOVERRIDE;
+
     // SetImageList helper
     void SetAnyImageList(wxImageList *imageList, int which);
 
@@ -221,20 +223,20 @@ protected:
     // end edit label
     void DoEndEditLabel(bool discardChanges = false);
 
-    virtual int DoGetItemState(const wxTreeItemId& item) const;
-    virtual void DoSetItemState(const wxTreeItemId& item, int state);
+    virtual int DoGetItemState(const wxTreeItemId& item) const wxOVERRIDE;
+    virtual void DoSetItemState(const wxTreeItemId& item, int state) wxOVERRIDE;
 
     virtual wxTreeItemId DoInsertItem(const wxTreeItemId& parent,
                                       size_t pos,
                                       const wxString& text,
                                       int image, int selectedImage,
-                                      wxTreeItemData *data);
+                                      wxTreeItemData *data) wxOVERRIDE;
     virtual wxTreeItemId DoInsertAfter(const wxTreeItemId& parent,
                                        const wxTreeItemId& idPrevious,
                                        const wxString& text,
                                        int image = -1, int selImage = -1,
-                                       wxTreeItemData *data = NULL);
-    virtual wxTreeItemId DoTreeHitTest(const wxPoint& point, int& flags) const;
+                                       wxTreeItemData *data = NULL) wxOVERRIDE;
+    virtual wxTreeItemId DoTreeHitTest(const wxPoint& point, int& flags) const wxOVERRIDE;
 
     // obtain the user data for the lParam member of TV_ITEM
     class wxTreeItemParam *GetItemParam(const wxTreeItemId& item) const;
@@ -261,6 +263,7 @@ protected:
     // return true if the key was processed, false otherwise
     bool MSWHandleSelectionKey(unsigned vkey);
 
+    virtual void MSWUpdateFontOnDPIChange(const wxSize& newDPI) wxOVERRIDE;
 
     // data used only while editing the item label:
     wxTextCtrl  *m_textCtrl;        // text control in which it is edited
@@ -297,6 +300,9 @@ private:
     // item visually spans the entire breadth of the window then
     bool MSWIsOnItem(unsigned flags) const;
 
+    // Delete the given item from the native control.
+    bool MSWDeleteItem(const wxTreeItemId& item);
+
 
     // the hash storing the items attributes (indexed by item ids)
     wxMapTreeAttr m_attrs;
@@ -312,6 +318,9 @@ private:
     // Virtual root item, if wxTR_HIDE_ROOT is set.
     void* m_pVirtualRoot;
 
+    // Item to call EnsureVisible() on when the tree is thawed, if necessary.
+    wxTreeItemId m_htEnsureVisibleOnThaw;
+
     // the starting item for selection with Shift
     wxTreeItemId m_htSelStart, m_htClickedItem;
     wxPoint m_ptClick;
@@ -335,7 +344,7 @@ private:
     friend class wxTreeItemIndirectData;
     friend class wxTreeSortHelper;
 
-    DECLARE_DYNAMIC_CLASS(wxTreeCtrl)
+    wxDECLARE_DYNAMIC_CLASS(wxTreeCtrl);
     wxDECLARE_NO_COPY_CLASS(wxTreeCtrl);
 };
 
