@@ -9,9 +9,6 @@
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
-#ifdef __BORLANDC__
-    #pragma hdrstop
-#endif
 
 // ----------------------------------------------------------------------------
 // NB: when using generic wxTimer implementation in your port, you *must* call
